@@ -1,18 +1,33 @@
 # -*- coding: utf-8 -*-
+
+"""
+Copyright (c) 2018 Keijack Wu
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in all
+copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+SOFTWARE.
+"""
+
+import http.cookies
+from typing import Dict, List, Union, Callable
 from simple_http_server.logger import get_logger
-try:
-    unicode("")
-except NameError:
-    # python 3 has no unicode type
-    unicode = str
-try:
-    import http.cookies as cookies
-except ImportError:
-    import Cookie as cookies
 
 name = "simple_http_server"
 
-
 __request_mappings = []
 __filters = []
 
@@ -27,7 +42,7 @@ def _log():
     return __logger
 
 
-def request_map(url, method=""):
+def request_map(url: str = "", method: Union[str, list] = "") -> Callable:
     def map(ctrl_fun):
         if isinstance(method, list):
             mths = method
@@ -45,7 +60,7 @@ def request_map(url, method=""):
     return map
 
 
-def filter_map(pattern):
+def filter_map(pattern: str = "") -> Callable:
     def map(filter_fun):
         __filters.append({"url_pattern": pattern, "func": filter_fun})
         return filter_fun
@@ -60,7 +75,11 @@ def _get_filters():
     return __filters
 
 
-class Request(object):
+class Cookies(http.cookies.SimpleCookie):
+    EXPIRE_DATE_FORMAT = "%a, %d %b %Y %H:%M:%S GMT"
+
+
+class Request:
     """Request"""
 
     def __init__(self):
@@ -76,35 +95,39 @@ class Request(object):
         self.json = None  # A dictionary if the `Content-Type` in request header is `application/json`
 
     @property
-    def cookies(self):
+    def cookies(self) -> Cookies:
         return self.__cookies
 
     @property
-    def parameters(self):
+    def parameters(self) -> Dict[str, List[str]]:
         return self.__parameters
 
     @parameters.setter
-    def parameters(self, val):
+    def parameters(self, val: Dict[str, List[str]]):
         self.__parameters = val
         self.__parameter = {}
         for k, v in self.__parameters.items():
             self.__parameter[k] = v[0]
 
     @property
-    def parameter(self):
+    def parameter(self) -> Dict[str, str]:
         return self.__parameter
 
-    def get_parameter(self, key, default=None):
+    def get_parameter(self, key: str, default: str = None) -> str:
         if key not in self.parameters.keys():
             return default
         else:
             return self.parameter[key]
 
 
-class MultipartFile(object):
+class MultipartFile:
     """Multipart file"""
 
-    def __init__(self, name="", required=False, filename="", content_type="", content=None):
+    def __init__(self, name: str = "",
+                 required: bool = False,
+                 filename: str = "",
+                 content_type: str = "",
+                 content: bytes = None):
         self.__name = name
         self.__required = required
         self.__filename = filename
@@ -112,100 +135,92 @@ class MultipartFile(object):
         self.__content = content
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.__name
 
     @property
-    def _required(self):
+    def _required(self) -> bool:
         return self.__required
 
     @property
-    def filename(self):
+    def filename(self) -> str:
         return self.__filename
 
     @property
-    def content_type(self):
+    def content_type(self) -> str:
         return self.__content_type
 
     @property
-    def content(self):
+    def content(self) -> bytes:
         return self.__content
 
     @property
-    def is_empty(self):
+    def is_empty(self) -> bool:
         return self.__content is None or len(self.__content) == 0
 
-    def save_to_file(self, file_path):
+    def save_to_file(self, file_path: str) -> None:
         if self.__content is not None and len(self.__content) > 0:
             with open(file_path, "wb") as f:
                 f.write(self.__content)
 
 
+class ParamStringValue(str):
 
-
-class ParamStringValue(unicode):
-
-    def __init__(self, name="", default="", required=False):
+    def __init__(self, name: str = "",
+                 default: str = "",
+                 required: bool = False):
         self.__name = name
         self.__required = required
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.__name
 
     @property
-    def _required(self):
+    def _required(self) -> bool:
         return self.__required
 
     def __new__(cls, name="", default="", **kwargs):
-        assert isinstance(default, str) or isinstance(default, unicode)
-        if str != unicode:
-            """
-            " Python 2.7, chuange str to unicode
-            """
-            default = default.decode("utf-8")
-        obj = super(ParamStringValue, cls).__new__(cls, default)
+        assert isinstance(default, str)
+        obj = super().__new__(cls, default)
         return obj
 
+
 class Parameter(ParamStringValue):
     pass
 
-class PathValue(unicode):
 
-    def __init__(self, name="", _value=""):
+class PathValue(str):
+
+    def __init__(self, name: str = "", _value: str = ""):
         self.__name = name
 
     @property
     def name(self):
         return self.__name
 
-    def __new__(cls, name="", _value="", **kwargs):
-        assert isinstance(_value, str) or isinstance(_value, unicode)
-        if str != unicode:
-            """
-            " Python 2.7, chuange str to unicode
-            """
-            _value = _value.decode("utf-8")
-        obj = super(PathValue, cls).__new__(cls, _value)
+    def __new__(cls, name: str = "", _value: str = "", **kwargs):
+        assert isinstance(_value, str)
+        obj = super().__new__(cls, _value)
         return obj
 
 
 class Parameters(list):
 
-    def __init__(self, name="", default=[], required=False):
+    def __init__(self, name: str = "", default: List[str] = [], required: bool = False):
         self.__name = name
         self.__required = required
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.__name
 
     @property
-    def _required(self):
+    def _required(self) -> bool:
         return self.__required
 
-    def __new__(cls, name="", default=[], **kwargs):
-        obj = super(Parameters, cls).__new__(cls)
+    def __new__(cls, name: str = "", default: List[str] = [], **kwargs):
+        obj = super().__new__(cls)
         obj.extend(default)
         return obj
 
@@ -223,13 +238,20 @@ class JSONBody(dict):
 """
 
 
-class Response(object):
+class StaticFile:
+
+    def __init__(self, file_path, content_type="application/octet-stream"):
+        self.file_path = file_path
+        self.content_type = content_type
+
+
+class Response:
     """Response"""
 
     def __init__(self,
-                 status_code=200,
-                 headers=None,
-                 body=""):
+                 status_code: int = 200,
+                 headers: Dict[str, str] = None,
+                 body: Union[str, dict, StaticFile, bytes] = ""):
         self.status_code = status_code
         self.__headers = headers if headers is not None else {}
         self.__body = ""
@@ -237,12 +259,12 @@ class Response(object):
         self.__set_body(body)
 
     @property
-    def cookies(self):
+    def cookies(self) -> http.cookies.SimpleCookie:
         return self.__cookies
 
     @cookies.setter
-    def cookies(self, val):
-        assert isinstance(val, cookies.SimpleCookie)
+    def cookies(self, val: http.cookies.SimpleCookie) -> None:
+        assert isinstance(val, http.cookies.SimpleCookie)
         self.__cookies = val
 
     @property
@@ -250,21 +272,21 @@ class Response(object):
         return self.__body
 
     @body.setter
-    def body(self, val):
+    def body(self, val: Union[str, dict, StaticFile, bytes]) -> None:
         self.__set_body(val)
 
     def __set_body(self, val):
-        assert val is None or type(val) in (str, unicode, dict, StaticFile, bytes), "Body type is not supported."
+        assert val is None or type(val) in (str, dict, StaticFile, bytes), "Body type is not supported."
         self.__body = val
 
     @property
-    def headers(self):
+    def headers(self) -> Dict[str, Union[list, str]]:
         return self.__headers
 
-    def set_header(self, key, value):
+    def set_header(self, key: str, value: str) -> None:
         self.__headers[key] = value
 
-    def add_header(self, key, value):
+    def add_header(self, key: str, value: Union[str, list]) -> None:
         if key not in self.__headers.keys():
             self.__headers[key] = value
             return
@@ -275,16 +297,16 @@ class Response(object):
         else:
             self.__headers[key].append(value)
 
-    def add_headers(self, headers={}):
+    def add_headers(self, headers: Dict[str, Union[str, List[str]]] = {}) -> None:
         if headers is not None:
             for k, v in headers.items():
                 self.add_header(k, v)
 
-    def send_error(self, status_code, message=""):
+    def send_error(self, status_code: int, message: str = ""):
         """abstruct method"""
         raise Exception("Abstruct method, you cannot call this method directly.")
 
-    def send_redirect(self, url):
+    def send_redirect(self, url: str):
         """abstruct method"""
         raise Exception("Abstruct method, you cannot call this method directly.")
 
@@ -295,25 +317,19 @@ class Response(object):
 
 class HttpError(Exception):
 
-    def __init__(self, code=400, message=""):
-        super(HttpError, self).__init__("HTTP_ERROR[%d] %s" % (code, message))
-        self.code = code
-        self.message = message
+    def __init__(self, code: int = 400, message: str = ""):
+        super().__init__("HTTP_ERROR[%d] %s" % (code, message))
+        self.code: int = code
+        self.message: str = message
 
 
-class StaticFile(object):
+class Redirect:
 
-    def __init__(self, file_path, content_type="application/octet-stream"):
-        self.file_path = file_path
-        self.content_type = content_type
-
-class Redirect(object):
-
-    def __init__(self, url):
+    def __init__(self, url: str):
         self.__url = url
 
     @property
-    def url(self):
+    def url(self) -> str:
         return self.__url
 
 
@@ -324,21 +340,18 @@ class Redirect(object):
 
 class Headers(dict):
 
-    def __init__(self, headers={}):
+    def __init__(self, headers: Dict[str, Union[str, List[str]]] = {}):
         self.update(headers)
 
 
-class Cookies(cookies.SimpleCookie):
-    EXPIRE_DATE_FORMAT = "%a, %d %b %Y %H:%M:%S GMT"
+class Cookie(http.cookies.Morsel):
 
-
-class Cookie(cookies.Morsel):
-
-    def __init__(self, name="",
-                 default="",
-                 default_options={},
-                 required=False):
-        super(Cookie, self).__init__()
+    def __init__(self,
+                 name: str = "",
+                 default: str = "",
+                 default_options: Dict[str, str] = {},
+                 required: bool = False):
+        super().__init__()
         self.__name = name
         self.__required = required
         if name is not None and name != "":
@@ -346,9 +359,9 @@ class Cookie(cookies.Morsel):
         self.update(default_options)
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.__name
 
     @property
-    def _required(self):
+    def _required(self) -> bool:
         return self.__required
