@@ -1,5 +1,42 @@
 # -*- coding: utf-8 -*-
 
+"""
+Copyright (c) 2018 Keijack Wu
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in all
+copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+SOFTWARE.
+"""
+
+import os
+import re
+import json
+import ssl as _ssl
+import inspect
+import threading
+import http.cookies as cookies
+import http.server
+
+from collections import OrderedDict
+from socketserver import ThreadingMixIn
+from urllib.parse import unquote
+from urllib.parse import quote
+from typing import Dict, Tuple
+
 from simple_http_server import HttpError
 from simple_http_server import StaticFile
 from simple_http_server import Headers
@@ -13,48 +50,9 @@ from simple_http_server import PathValue
 from simple_http_server import Parameter
 from simple_http_server import MultipartFile
 from simple_http_server import Request
-import sys
-import os
-import re
-import json
-import copy
-import inspect
-from collections import OrderedDict
-import threading
-try:
-    import http.cookies as cookies
-except ImportError:
-    import Cookie as cookies
-try:
-    import http.server as BaseHTTPServer
-except ImportError:
-    import BaseHTTPServer
-try:
-    from socketserver import ThreadingMixIn
-except ImportError:
-    from SocketServer import ThreadingMixIn
-try:
-    from urllib.parse import unquote
-except ImportError:
-    from urllib import unquote
-try:
-    from urllib.parse import quote
-except ImportError:
-    from urllib import quote
-try:
-    long(0)
-except NameError:
-    # python 3 does no longer support long method, use int instead
-    long = int
-try:
-    unicode("")
-except NameError:
-    # python 3 does no longer support unicode method, use str instead
-    unicode = str
 
 from simple_http_server.logger import get_logger
 
-
 _logger = get_logger("simple_http_server.http_server")
 
 
@@ -78,22 +76,22 @@ class ResponseWrapper(Response):
         self.__send_lock__ = threading.Lock()
 
     @property
-    def is_sent(self):
+    def is_sent(self) -> bool:
         return self.__is_sent
 
-    def send_error(self, status_code, message=""):
+    def send_error(self, status_code: int, message: str = "") -> None:
         self.status_code = status_code
         msg = message if message is not None else ""
         self.body = {"error": msg}
         self.send_response()
 
-    def send_redirect(self, url):
+    def send_redirect(self, url: str) -> None:
         self.status_code = 302
         self.set_header("Location", url)
         self.body = None
         self.send_response()
 
-    def send_response(self):
+    def send_response(self) -> None:
         with self.__send_lock__:
             self.__send_response()
 
@@ -118,11 +116,11 @@ class FilterContex(object):
         self.__filters = filters if filters is not None else []
 
     @property
-    def request(self):
+    def request(self) -> Request:
         return self.__request
 
     @property
-    def response(self):
+    def response(self) -> Response:
         return self.__response
 
     def do_chain(self):
@@ -181,7 +179,7 @@ class FilterContex(object):
                 headers.update(item)
             elif isinstance(item, cookies.BaseCookie):
                 cks.update(item)
-            elif type(item) in (str, unicode, dict, StaticFile, bytes):
+            elif type(item) in (str, dict, StaticFile, bytes):
                 if body is None:
                     body = item
         return status_code, headers, cks, body
@@ -226,14 +224,12 @@ class FilterContex(object):
                 kwarg_vals[k] = self.__build_params(k, v)
             elif isinstance(v, JSONBody):
                 kwarg_vals[k] = self.__build_json_body()
-            elif isinstance(v, str) or isinstance(v, unicode):
+            elif isinstance(v, str):
                 kwarg_vals[k] = self.__build_str(k, v)
             elif isinstance(v, bool):
                 kwarg_vals[k] = self.__build_bool(k, v)
             elif isinstance(v, int):
                 kwarg_vals[k] = self.__build_int(k, v)
-            elif isinstance(v, long):
-                kwarg_vals[k] = self.__build_long(k, v)
             elif isinstance(v, list):
                 kwarg_vals[k] = self.__build_list(k, v)
             elif isinstance(v, dict):
@@ -291,15 +287,6 @@ class FilterContex(object):
         else:
             return val
 
-    def __build_long(self, key, val=0):
-        if key in self.request.parameter.keys():
-            try:
-                return long(self.request.parameter[key])
-            except:
-                raise HttpError(400, "Parameter[%s] should be an int. " % key)
-        else:
-            return val
-
     def __build_int(self, key, val=0):
         if key in self.request.parameter.keys():
             try:
@@ -350,11 +337,6 @@ class FilterContex(object):
 
     def __build_param(self, key, val=Parameter()):
         name = val.name if val.name is not None and val.name != "" else key
-        if not isinstance(name, unicode):
-            """
-            " Python 2.7, change str => unicode, or it will fail to reconize the key that is unicode;
-            """
-            name = name.decode("utf-8")
         if val._required and name not in self.request.parameter:
             raise HttpError(400, "Parameter[%s] is required." % name)
         if name in self.request.parameter:
@@ -387,7 +369,7 @@ def _remove_url_first_slash(url):
     return _url
 
 
-class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+class _SimpleDispatcherHttpRequestHandler(http.server.BaseHTTPRequestHandler):
     """The Class will dispatch the request to the controller configured in RequestMapping"""
 
     def __process(self, method):
@@ -468,15 +450,15 @@ class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler)
                     dic[k].append(i)
         return dic
 
-    def __get_query_string(self, oriPath):
-        parts = oriPath.split('?')
+    def __get_query_string(self, ori_path):
+        parts = ori_path.split('?')
         if len(parts) == 2:
             return parts[1]
         else:
             return ""
 
-    def __get_path(self, oriPath):
-        path = oriPath.split('?', 1)[0]
+    def __get_path(self, ori_path):
+        path = ori_path.split('?', 1)[0]
         path = path.split('#', 1)[0]
         path = _remove_url_first_slash(path)
         return path
@@ -541,13 +523,7 @@ class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler)
             return params
         pairs = query_string.split("&")
         for item in pairs:
-            """
-            " for python 2.7: val here is a unicode, after unquote,
-            " it still is a unicode, and may cause a encoding problem,
-            " so here we fource to change it into a str
-            """
-            str_item = str(item)
-            key, val = self.__break(str_item, "=")
+            key, val = self.__break(item, "=")
             if val is None:
                 val = ""
             self.__put_to(params, unquote(key), unquote(val))
@@ -574,11 +550,11 @@ class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler)
         self.send_response(status_code)
         self.send_header("Last-Modified", str(self.date_time_string()))
         for k, v in headers.items():
-            if isinstance(v, str) or isinstance(v, unicode):
+            if isinstance(v, str):
                 self.send_header(k, v)
             elif isinstance(v, list):
                 for iov in v:
-                    if isinstance(iov, str) or isinstance(iov, unicode):
+                    if isinstance(iov, str):
                         self.send_header(k, iov)
 
         for k in cks:
@@ -588,7 +564,7 @@ class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler)
         if body is None:
             self.send_header("Content-Length", 0)
             self.end_headers()
-        elif isinstance(body, str) or isinstance(body, unicode):
+        elif isinstance(body, str):
             try:
                 data = body.encode("utf-8")
                 self.send_header("Content-Length", len(data))
@@ -621,7 +597,7 @@ class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler)
         elif isinstance(raw_body, dict):
             content_type = "application/json; charset=utf8"
             body = json.dumps(raw_body, ensure_ascii=False)
-        elif isinstance(raw_body, str) or isinstance(raw_body, unicode):
+        elif isinstance(raw_body, str):
             body = raw_body.strip()
             if body.startswith("<?xml") and body.endswith(">"):
                 content_type = "text/xml; charset=utf8"
@@ -637,6 +613,8 @@ class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler)
         elif isinstance(raw_body, bytes):
             body = raw_body
             content_type = "application/octet-stream"
+        else:
+            body = raw_body
         return content_type, body
 
     def do_method(self, method):
@@ -671,7 +649,7 @@ class _SimpleDispatcherHttpRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler)
         _logger.info("%s -  %s" % (self.client_address[0], format % args))
 
 
-class _HttpServerWrapper(BaseHTTPServer.HTTPServer, object):
+class _HttpServerWrapper(http.server.HTTPServer, object):
 
     HTTP_METHODS = ["OPTIONS", "GET", "HEAD", "POST", "PUT", "DELETE", "TRACE", "CONNECT"]
 
@@ -717,8 +695,6 @@ class _HttpServerWrapper(BaseHTTPServer.HTTPServer, object):
 
     def __get_path_reg_pattern(self, url):
         _url = url
-        if not isinstance(_url, unicode):
-            _url = _url.decode("utf-8")
         path_names = re.findall("(?u)\\{\\w+\\}", _url)
         if len(path_names) == 0:
             # normal url
@@ -730,8 +706,6 @@ class _HttpServerWrapper(BaseHTTPServer.HTTPServer, object):
         quoted_names = []
         for name in path_names:
             name = name[1: -1]
-            if str != unicode:
-                name = name.encode("utf-8")
             quoted_names.append(quote(name))
         return _url, quoted_names
 
@@ -772,7 +746,7 @@ class _HttpServerWrapper(BaseHTTPServer.HTTPServer, object):
         elif ext == ".pdf":
             content_type = "application/pdf"
         elif ext == ".mp4":
-            content_type = "video/mpeg4"
+            content_type = "video/mp4"
         elif ext == ".mp3":
             content_type = "audio/mp3"
         else:
@@ -801,8 +775,6 @@ class _HttpServerWrapper(BaseHTTPServer.HTTPServer, object):
 
     def __try_get_from_path_val(self, path, method):
         for patterns, val in self.path_val_url_mapping[method].items():
-            if not isinstance(patterns, unicode):
-                patterns = patterns.decode("utf-8")
             m = re.match(patterns, path)
             is_match = m is not None
             _logger.debug("pattern::[%s] => path::[%s] match? %s" % (patterns, path, str(is_match)))
@@ -839,19 +811,47 @@ class SimpleDispatcherHttpServer(object):
     def map_request(self, url, fun, method=""):
         self.server.map_url(url, fun, method)
 
-    def __init__(self, host=('', 9090), multithread=True, resources={}):
+    def __init__(self,
+                 host: Tuple[str, int] = ('', 9090),
+                 ssl: bool = False,
+                 ssl_protocol: int = _ssl.PROTOCOL_TLS_SERVER,
+                 ssl_check_hostname: bool = False,
+                 keyfile: str = "",
+                 certfile: str = "",
+                 keypass: str = "",
+                 multithread: bool = True,
+                 ssl_context: _ssl.SSLContext = None,
+                 resources: Dict[str, str] = {}):
         self.host = host
         self.multithread = multithread
+        self.ssl = ssl
         if self.multithread:
             self.server = _ThreadingHttpServer(self.host, res_conf=resources)
         else:
             self.server = _HttpServerWrapper(self.host, res_conf=resources)
 
+        if ssl:
+            if ssl_context:
+                ssl_ctx = ssl_context
+            else:
+                assert keyfile and certfile, "keyfile and certfile should be provided. "
+                ssl_ctx = _ssl.SSLContext(protocol=ssl_protocol)
+                ssl_ctx.check_hostname = ssl_check_hostname
+                ssl_ctx.load_cert_chain(certfile=certfile, keyfile=keyfile, password=keypass)
+            self.server.socket = ssl_ctx.wrap_socket(
+                self.server.socket,
+                server_side=True
+            )
+
     def resources(self, res={}):
         self.server.res_conf = res
 
     def start(self):
-        _logger.info("Dispatcher Http Server starts. Listen to port [" + str(self.host[1]) + "]")
+        if self.ssl:
+            ssl_hint = " with SSL on"
+        else:
+            ssl_hint = ""
+        _logger.info("Dispatcher Http Server starts. Listen to port [%d]%s." % (self.host[1], ssl_hint))
         self.server.serve_forever()
 
     def shutdown(self):
